program XENUGraph;

{$APPTYPE CONSOLE}

uses
  SysUtils;

type
  TXENUEntry = record
    OriginPage       : String;
    LinkToPage       : String;
    LinkToPageStatus : String;
    LinkToPageTitle  : String;
    OriginPageDate   : String;
  end;

function GenerateFile : Boolean;
  function CheckParameters(var AInputFile : String; var AOutputFile : String; var ARootURL : String) : Boolean;
  begin
    Result := false;

    if (ParamCount >= 3) then
    begin
      AInputFile  := AnsiLowerCase(Trim(ParamStr(1)));
      AOutputFile := AnsiLowerCase(Trim(ParamStr(2)));
      ARootURL    := AnsiLowerCase(Trim(ParamStr(3)));

      Result := FileExists(AInputFile);
    end;
  end;

  function EqualXENUEntries(const AEntryA : TXENUEntry; const AEntryB : TXENUEntry) : Boolean;
  begin
    Result := (AnsiLowerCase(Trim(AEntryA.OriginPage)) = AnsiLowerCase(Trim(AEntryB.OriginPage))) and
              (AnsiLowerCase(Trim(AEntryA.LinkToPage)) = AnsiLowerCase(Trim(AEntryB.LinkToPage))) and
              (AnsiLowerCase(Trim(AEntryA.LinkToPageStatus)) = AnsiLowerCase(Trim(AEntryB.LinkToPageStatus))) and
              (AnsiLowerCase(Trim(AEntryA.LinkToPageTitle)) = AnsiLowerCase(Trim(AEntryB.LinkToPageTitle))) and
              (AnsiLowerCase(Trim(AEntryA.OriginPageDate)) = AnsiLowerCase(Trim(AEntryB.OriginPageDate)));
  end;

  function ParseXENUEntry(ALine : String) : TXENUEntry;
    function NextItem(var ALine : String) : String;
    const
      CDivider = #9;
    begin
      Result := '';

      ALine := Trim(ALine);
      if (Pos(CDivider, ALine) > 0) then
      begin
        Result := Copy(ALine, 1, Pred(Pos(CDivider, ALine)));
        ALine  := Copy(ALine, Succ(Pos(CDivider, ALine)), Length(ALine) - Pos(CDivider, ALine));
      end
      else
      begin
        Result := ALine;
        ALine  := '';
      end;
    end;
  begin
    Result.OriginPage       := NextItem(ALine);
    Result.LinkToPage       := NextItem(ALine);
    Result.LinkToPageStatus := NextItem(ALine);
    Result.LinkToPageTitle  := NextItem(ALine);
    Result.OriginPageDate   := NextItem(ALine);
  end;

  procedure WriteHeader;
  begin
    WriteLn('XENUGraph 0.1b1');
    WriteLn('(C) 2008 by coltishWARE (Kevin Niehage)');
    WriteLn('');
  end;

  procedure WriteHelp;
  begin
    Write('> ');
    Write(ChangeFileExt(ExtractFileName(ParamStr(0)), ''));
    WriteLn(' [INPUTFILE] [OUTPUTFILE] [ROOTURL]');

    WriteLn('');
    WriteLn('This tool will convert a TAB seperared Page Map file generated by XENU''s Link Sleuth into a GraphViz graph description.');
    WriteLn('');
    WriteLn('XENU''s Link Sleuth can be found at http://home.snafu.de/tilman/xenulink.html');
    WriteLn('GraphViz can be found at http://www.graphviz.org/');
    WriteLn('');
  end;

const
  CHeader : TXENUEntry = (OriginPage       : 'OriginPage';
                          LinkToPage       : 'LinkToPage';
                          LinkToPageStatus : 'LinkToPageStatus';
                          LinkToPageTitle  : 'LinkToPageTitle';
                          OriginPageDate   : 'OriginPageDate');
var
  LDestination : String;
  LInput       : TextFile;
  LInputFile   : String;
  LNextLine    : String;
  LOutput      : TextFile;
  LOutputFile  : String;
  LRootURL     : String;
  LSource      : String;
  LXENUEntry   : TXENUEntry;
begin
  Result := false;

  WriteHeader;

  // current status
  WriteLn('-> Checking Parameters');

  if CheckParameters(LInputFile, LOutputFile, LRootURL) then
  begin
    // current status
    WriteLn('-> Parameters Accepted');
    WriteLn('-> Opening Input File');

    AssignFile(LInput, LInputFile);

    Reset(LInput);
    try
      // current status
      WriteLn('-> Input File Opened');
      WriteLn('-> Reading XENU Header');

      // first check for correct XENU header
      ReadLn(LInput, LNextLine);
      LXENUEntry := ParseXENUEntry(LNextLine);
      if EqualXENUEntries(LXENUEntry, CHeader) then
      begin
        // current status
        WriteLn('-> XENU Header Accepted');
        WriteLn('-> Opening Output File');

        AssignFile(LOutput, LOutputFile);

        Rewrite(LOutput);
        try
          // current status
          WriteLn('-> Output File Opened');
          WriteLn('-> Writing GraphViz Header');

          WriteLn(LOutput, 'digraph G');
          WriteLn(LOutput, '{');

          // current status
          WriteLn('-> GraphViz Header Written');
          WriteLn('-> Writing GraphViz Entries');

          while not(EoF(LInput)) do
          begin
            // read next XENU entry
            ReadLn(LInput, LNextLine);
            LXENUEntry := ParseXENUEntry(LNextLine);

            LDestination := Trim(LXENUEntry.LinkToPage);
            if (Pos(LRootURL, LDestination) = 1) then
            begin
              Delete(LDestination, 1, Length(LRootURL));
              LDestination := Trim(LDestination);

              if (LDestination = '') then
                LDestination := LRootURL;
            end;

            LSource := Trim(LXENUEntry.OriginPage);
            if (Pos(LRootURL, LSource) = 1) then
            begin
              Delete(LSource, 1, Length(LRootURL));
              LSource := Trim(LSource);

              if (LSource = '') then
                LSource := LRootURL;
            end;

            Write(LOutput, '  "');
            Write(LOutput, LSource);
            Write(LOutput, '" -> "');
            Write(LOutput, LDestination);
            WriteLn(LOutput, '";');
          end;

          // current status
          WriteLn('-> GraphViz Entries Written');
          WriteLn('-> Writing GraphViz Footer');

          WriteLn(LOutput, '}');

          // current status
          WriteLn('-> GraphViz Footer Written');
          WriteLn('=> Done');

          Result := true;
        finally
          Close(LOutput);
        end;
      end
      else
      begin
        // current status
        WriteLn('-> XENU Header Not Accepted');
      end;
    finally
      Close(LInput);
    end;
  end
  else
  begin
    // current status
    WriteLn('-> Parameters Not Accepted');
  end;

  if not(Result) then
    WriteHelp;
end;

begin
  GenerateFile;
end.
